import * as React from 'react';
import { get, set } from 'lodash';

import {FormContext} from './FormContext';
import { DataTypeSingle } from '../types';

export interface FormProviderProps<DataType> extends React.PropsWithChildren {
  defaultValues?: Partial<DataType>;
}

const FormProvider = <DataType extends object>({
  children,
  defaultValues = {},
}: FormProviderProps<DataType>) => {
  const [data, setData] = React.useState<DataType>(defaultValues as DataType);
  const [errors, setErrors] = React.useState<Record<string, string>>({});

  const getData = React.useCallback(() => {
    return data;
  }, [data]);

  // This function updates a specific field in the form data.
  // It takes a field key and a new value as parameters.
  const updateField = React.useCallback(
    (fieldKey: string, value: never) => {
      // Create a copy of the current form data.
      const newData = { ...getData() };
      // Use the 'set' function to update the value of the specified field in the copied data.
      set(newData, fieldKey, value);
      // Update the form data with the modified copy.
      setData(newData);
    },
    [setData, getData]
  );

  const getFieldValue = React.useCallback(
    (fieldKey: string) => {
      return get(data, fieldKey) as DataTypeSingle;
    },
    [data]
  );

  const getError = React.useCallback(
    (fieldKey: string) => {
      return get(errors, fieldKey);
    },
    [errors]
  );

  return (
      <FormContext.Provider
          value={{ getFieldValue, getData, updateField, data, setData, errors, setErrors, getError }}
      >
        {children}
      </FormContext.Provider>
  );
};

export default FormProvider;
